package com.academiaandroid.tareassegundoplano;

//Ejemplo aplicacin Android en el que realizaremos la programacin de tareas en segundo plano
//haciendo uso de las Clases Thread, Handler y AlarmManager
//
//academiaandroid.com
//
//by Jos Antonio Gzquez Rodrguez


import java.util.Calendar;
import android.app.Activity;
import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.TimePicker;
import android.widget.Toast;

public class MainActivity extends Activity {
	
	//Se definen los elementos necesarios para la lgica de 
	//manejo de la aplicacin.
	private TextView tvTipoTarea;
	private static Button btnArrancarHilo;
	private Button btnReiniciar;
	private Button btnArrancarAsyncTask;
	private Button btnDetenerAsyncTask;	
	private ImageButton btnProgramarTarea;
	private ImageButton btnProgramarTareaRepetitiva;
	private TimePicker timePickerTarea;
	ProgressBar prgssTarea;
	private TextView tvEstadoProgreso;	
	private EditText edSegundos;
	TareaAsyncTask tareasynctask;
	ImageView imgAA;
	AlarmManager planificarAlarma;
	private static MainActivity mainactivity;
	
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        mainactivity = this;
        
        //Enlazamos los componentes con sus recursos a nivel de Layout, adems 
        //de establecer la lgica de visibilidad de los botones.
        tvTipoTarea = (TextView)findViewById(R.id.tvTipo);
        btnArrancarHilo = (Button)findViewById(R.id.btnArrancarHilo);
        btnReiniciar = (Button)findViewById(R.id.btnReiniciar);
        btnArrancarAsyncTask = (Button)findViewById(R.id.btnArrancarAsyncTask);
        btnDetenerAsyncTask = (Button)findViewById(R.id.btnDetenerAsyncTask);        
        prgssTarea = (ProgressBar)findViewById(R.id.prgssTarea); 
        imgAA = (ImageView)findViewById(R.id.imgAA);
        tvEstadoProgreso = (TextView)findViewById(R.id.tvEstadoProgreso);
		tvEstadoProgreso.setTextColor(Color.RED);
		tvTipoTarea.setTextColor(Color.RED);
		btnReiniciar.setVisibility(View.INVISIBLE);
		btnDetenerAsyncTask.setVisibility(View.INVISIBLE);
		timePickerTarea = (TimePicker)findViewById(R.id.timePickerTarea);
		btnProgramarTarea = (ImageButton)findViewById(R.id.btnAlarmManager);
		btnProgramarTareaRepetitiva = (ImageButton)findViewById(R.id.btnProgramarTareaRepetitiva);
		edSegundos = (EditText)findViewById(R.id.edSegundos);
		
		//Evento que caza el cambio de estado del botn 'Arrancar AsyncTask',
		//mostrando que accin ha sido lanzada, y llamando al mtodo 'execute()'
		//pasndole como parmetros la URL de la imagen a descargar.
        btnArrancarAsyncTask.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				tvTipoTarea.setTextColor(Color.RED);
				tvTipoTarea.setText(" AsyncTask");	
				tareasynctask = new TareaAsyncTask(MainActivity.this, imgAA);
				tareasynctask.execute("https://dl.dropboxusercontent.com/u/67598004/LogoVertical_baja.png");
				btnDetenerAsyncTask.setVisibility(View.VISIBLE);				
			}
		});
        
        //Evento que caza el cambio de estado del botn 'Detener AsyncTask',
        //para detener la tarea realizada en segundo plano, por la Clase AsyncTask,
        //llamando a su mtodo 'cancel()',permitiendo interrumpir su funcionamiento.
        btnDetenerAsyncTask.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {			
				tareasynctask = new TareaAsyncTask(MainActivity.this, imgAA);
				tareasynctask.cancel(true);
				btnDetenerAsyncTask.setVisibility(View.INVISIBLE);
			}
		});
        
        //Evento que caza el cambio de estado del botn 'Arrancar Hilo + Handler',
        //para realizar la llamada al mtodo 'runThread()', encargado de
        //realizar las tareas en segundo plano, mediante el uso de las Clases
        //Thread y Handler.
        btnArrancarHilo.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {	
				Toast.makeText(getApplicationContext(), "Nueva Tarea lanzada!!!", Toast.LENGTH_SHORT).show();
				tvTipoTarea.setTextColor(Color.RED);
				tvTipoTarea.setText(" Thread");							
				runThread();
						}			
		});   
        
        //Evento que caza el cambio de estado del botn 'Reiniciar Barra de Progreso',
        //para que no sea visible el botn y mostrar informacin sobre el estado
        //del componente ProgressBar, en este caso 'Sin Progreso'.
        btnReiniciar.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				reiniciarBarraProgreso();
				btnReiniciar.setVisibility(View.INVISIBLE);
				tvEstadoProgreso.setTextColor(Color.RED);				
				tvEstadoProgreso.setText(" Sin Progreso");
			}
		});
        
        //Evento que caza el cambio de estado del botn con la imagen del reloj,
        //estableciendo la hora de arranque en la hora actual, instanciando la clase Calendar,
        //y programando la tarea para ser ejecutada a la hora indicada por el usuario.
        btnProgramarTarea.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {	
				
				try{
					Calendar cal = Calendar.getInstance(); 
					cal.setTimeInMillis(System.currentTimeMillis());
					cal.set (Calendar.HOUR_OF_DAY, timePickerTarea.getCurrentHour()); 
					cal.set (Calendar.MINUTE, timePickerTarea.getCurrentMinute()); 
					cal.set (Calendar.SECOND, 0);
					
					int hora = timePickerTarea.getCurrentHour();
					int minutos = timePickerTarea.getCurrentMinute();
					
					String hora_programada = new StringBuilder().append(hora).append(":0").append(minutos) + " hrs";
					if(minutos < 10)
					{
						tvTipoTarea.setText("Tarea programada a las --> " + hora_programada);
					}else
					{
						hora_programada = new StringBuilder().append(hora).append(":").append(minutos) + " hrs";
						tvTipoTarea.setText("Tarea programada a las --> " + hora_programada);
					}				
					Toast.makeText(getApplicationContext(), "Tarea programada a las " + hora_programada.toString(), Toast.LENGTH_SHORT).show();
					planificarAlarma = (AlarmManager)getSystemService(ALARM_SERVICE);
					Intent intent = new Intent(getApplicationContext(), PlanificarTareas.class);
					PendingIntent pi = PendingIntent.getBroadcast(getApplicationContext(), 0, intent, 0);
					planificarAlarma.set(AlarmManager.RTC_WAKEUP, cal.getTimeInMillis(), pi);
				}catch(Exception ex)
				{
					System.out.println("Error al programar hora de tarea: " + ex.getMessage());
				}
			}
		});
        
        //Evento que caza el cambio de estado del botn con la imagen de dos flechas a modo de recarga,
        //estableciendo como parmetro de repeticin de la alarma, los segundos introducidos en el componente 
        //EditText.
        btnProgramarTareaRepetitiva.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				if(edSegundos.getText().toString().equals(""))
				{
					Toast.makeText(getApplicationContext(), "Debe indicar el intervalo de tiempo para la repeticin de las tareas.", Toast.LENGTH_LONG).show();
				}else
				{
					String segundos = edSegundos.getText().toString();
					tvTipoTarea.setText("Repeticin de tarea cada " + segundos + " sg.");
					Toast.makeText(getApplicationContext(), "Tarea programada cada " + segundos + " sg.", Toast.LENGTH_SHORT).show();
					planificarAlarma = (AlarmManager)getSystemService(ALARM_SERVICE);
					Intent intent = new Intent(getApplicationContext(), PlanificarTareas.class);
					PendingIntent pi = PendingIntent.getBroadcast(getApplicationContext(), 0, intent, 0);				
					long tiempo_repeticion = Long.parseLong(segundos);
					planificarAlarma.setRepeating(AlarmManager.ELAPSED_REALTIME_WAKEUP, 60000, tiempo_repeticion*1000, pi);
				}
			}
		});
    }
    
    public static MainActivity  getInstance()
    {
         return mainactivity;
    }
    
    //Mtodo que permite poner a 0 el componente ProgressBar definido.
    public void reiniciarBarraProgreso()
    {
    	try
    	{
    		prgssTarea.setProgress(0);
    		Toast.makeText(getApplicationContext(), "Barra de Progreso reiniciada", Toast.LENGTH_LONG).show();
    		
    	}catch(Exception ex)
    	{
    		Toast.makeText(getApplicationContext(), "Error al reiniciar las imgenes a valor no visible.", Toast.LENGTH_LONG).show();
    	}
    }
    
    //Mtodo dnde se define el tiempo en milisegundos de pausa del hilo. 
    public void tiempoTarea()
    {
    	try {
			Thread.sleep(100);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
    }
    
    //Mtodo encargado de actualizar el porcentaje de la barra de progreso.
    private Handler handler = new Handler()
    {
		int prg = 0;
    	@Override
        public void handleMessage(Message msg) 
        {
            prg ++;
            prgssTarea.setProgress(prg);
            String porcentaje = String.valueOf(prg).toString();
            tvEstadoProgreso.setTextColor(Color.RED);
            tvEstadoProgreso.setText(" " + porcentaje + "% completado");
            if(porcentaje.equals("100"))
            {
            	prg = 0;
            }
        }
    };

    //Mtodo donde se define la lgica de ejecucin de un hilo,
    //simulando una barra de progreso realizando una tarea.
    public void runThread() {
		new Thread(){
			public void run(){
				runOnUiThread(new Runnable() {
					
					@Override
					public void run() {					
						prgssTarea.setProgress(0);						
					}			
				});
				
				for(int i=1; i<=100; i++) {

						tiempoTarea();	
						prgssTarea.post(new Runnable() {
						public void run() 
						{			
							handler.sendMessage(handler.obtainMessage());
						}												
					});											
				}
				
				runOnUiThread(new Runnable() {
					
					@Override
					public void run() {	
						tvEstadoProgreso.setText(" Finalizado");
						btnReiniciar.setVisibility(View.VISIBLE);
						Toast.makeText(getApplicationContext(), "Tarea con hilos finalizada", Toast.LENGTH_SHORT).show();						
					}
				});
			}
		}.start();				
	} 
}


