package com.academiaandroid.tareassegundoplano;

//Ejemplo aplicacin Android en el que realizaremos tareas en segundo plano
//haciendo uso de las Clases Thread, Handler y AsyncTask
//
//academiaandroid.com
//
//by Jos Antonio Gzquez Rodrguez

import java.io.BufferedInputStream;
import java.lang.ref.WeakReference;
import java.net.URL;
import java.net.URLConnection;
import android.app.ProgressDialog;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.ImageView;
import android.widget.Toast;

//Creamos una clase que herede de la clase base AsyncTask con los
//tipos de parmetro de entrada, parmetro para la actualizacin del progreso
//en el hilo principal y el resultado devuelto del procesamiento.
public class TareaAsyncTask extends AsyncTask<String, Integer, Bitmap>
{
	//Se declaran los elementos necesarios para la descarga y
	//visionado de la imagen.
	private MainActivity context;
	private final WeakReference imageRef;
	private BufferedInputStream input;
	private Bitmap bitmap;
	private ImageView imgAA;
	private ProgressDialog progressDialog;
	
	public TareaAsyncTask(MainActivity context, ImageView imgAA){
	    this.context = context;
	    imageRef = new WeakReference(imgAA);
	}
	//Mtodo que ejecutar las tareas en segundo plano, encargado de conectarse 
	//a una direccin URL con la imagen que se va a descargar.
	@Override
	protected Bitmap doInBackground(String... params) {
		try{						
			URL url = new URL(params[0]);
            URLConnection conection = url.openConnection();
            conection.connect();
            input = new BufferedInputStream(url.openStream(), 8192);
            bitmap = BitmapFactory.decodeStream(input);
            input.close();
		}catch(Exception ex)
		{
			Toast.makeText(context, "Error en doInBackground: " + ex.getMessage(), Toast.LENGTH_LONG).show();
			Log.e("Error en doInBackground: ", ex.getMessage());
		}
		return bitmap;
	}
	
	//Mtodo que se ejecutar antes del procesamiento de la tarea, 
	//indicando en un componente ProgressDialog,
	//que las tareas en segundo plano van a comenzar a ejecutarse
	@Override
    protected void onPreExecute() {
		try{
			super.onPreExecute();
			progressDialog = ProgressDialog.show(context, "Espere", "Descargando archivo...");
			progressDialog.setCancelable(true);
			}catch(Exception ex)
			{
				Toast.makeText(context, "Error en onPreExecute: " + ex.getMessage(), Toast.LENGTH_LONG).show();
				Log.e("Error en onPreExecute: ", ex.getMessage());
			}
    }
	
	//Mtodo que se procesar justo despus de ejecutarse la tarea en 
	//segundo plano, encargado de mostrar la imagen descargada en la 
	//pantalla.
	@Override
	public void onPostExecute(Bitmap bitmap)
	    {
			try{
				
				progressDialog.dismiss();
				if(isCancelled())
				{
					bitmap = null;
				}
				if (imageRef != null) {
		            ImageView imageView = (ImageView) imageRef.get();
		            if (imageView != null && bitmap != null) {
		                imageView.setImageBitmap(bitmap);	
		                Toast.makeText(context, "Mostrando imagen descargada", Toast.LENGTH_LONG).show();
		            } else
		                Toast.makeText(context, "Error al descargar la imagen", Toast.LENGTH_LONG).show();
		        }
			}catch(Exception ex)
			{
				Toast.makeText(context, "Error en onPostExecute: " + ex.getMessage(), Toast.LENGTH_LONG).show();
				Log.e("Error en onPostExecute: ", ex.getMessage());
			}
	    }
	
	//Mtodo encargado de cancelar la ejecucin de la tarea.
	@Override
	protected void onCancelled() {
			ImageView imageView = (ImageView) imageRef.get();
			imageView.setImageBitmap(null);
			
	        Toast.makeText(context, "Tarea con 'AsyncTask' detenida por el usuario",Toast.LENGTH_SHORT).show();        
	    }

}
